@@ -136,7 +136,16 @@ class AudioSegment(object):
                 setattr(self, attr, val)
         else:
             # normal construction
-            data = data if isinstance(data, (basestring, bytes)) else data.read()
+            try:
+                data = data if isinstance(data, (basestring, bytes)) else data.read()
+            except(OSError):
+                d = b''
+                reader = data.read(2**31-1)
+                while reader:
+                    d += reader
+                    reader = data.read(2**31-1)
+                data = d
+
             raw = wave.open(StringIO(data), 'rb')
 
             raw.rewind()
@@ -320,6 +329,12 @@ class AudioSegment(object):
         if isinstance(data, list):
             data = b''.join(data)
 
+        if isinstance(data, array.array):
+            try:
+                data = data.tobytes()
+            except:
+                data = data.tostring()
+
         # accept file-like objects
         if hasattr(data, 'read'):
             if hasattr(data, 'seek'):
@@ -417,7 +432,17 @@ class AudioSegment(object):
             return cls(data=file.read(), metadata=metadata)
 
         input_file = NamedTemporaryFile(mode='wb', delete=False)
-        input_file.write(file.read())
+        try:
+            input_file.write(file.read())
+        except(OSError):
+            input_file.flush()
+            input_file.close()
+            input_file = NamedTemporaryFile(mode='wb', delete=False, buffering=2**31-1)
+            file = open(orig_file, buffering=2**13-1, mode='rb')
+            reader = file.read(2**31-1)
+            while reader:
+                input_file.write(reader)
+                reader = file.read(2**31-1)
         input_file.flush()
 
         output = NamedTemporaryFile(mode="rb", delete=False)
@@ -481,7 +506,7 @@ class AudioSegment(object):
         file.seek(0)
         return cls(data=file)
 
-    def export(self, out_f=None, format='mp3', codec=None, bitrate=None, parameters=None, tags=None, id3v2_version='4'):
+    def export(self, out_f=None, format='mp3', codec=None, bitrate=None, parameters=None, tags=None, id3v2_version='4', cover=None):
         """
         Export an AudioSegment to a file with given options
 
@@ -510,6 +535,9 @@ class AudioSegment(object):
 
         id3v2_version (string)
             Set ID3v2 version for tags. (default: '4')
+
+        cover (file)
+            Set cover for audio file from image file. (png or jpg)
         """
         id3v2_allowed_versions = ['3', '4']
 
@@ -553,6 +581,12 @@ class AudioSegment(object):
         if codec is None:
             codec = self.DEFAULT_CODECS.get(format, None)
 
+        if cover is not None:
+            if cover.lower().endswith(('.png', '.jpg', '.jpeg', '.bmp', '.tif', '.tiff')) and format == "mp3":
+                conversion_command.extend(["-i" , cover, "-map", "0", "-map", "1", "-c:v", "mjpeg"])
+            else:
+                raise AttributeError("Currently cover images are only supported by MP3 files. The allowed image formats are: .tif, .jpg, .bmp, .jpeg and .png.")
+
         if codec is not None:
             # force audio encoder
             conversion_command.extend(["-acodec", codec])
@@ -597,7 +631,7 @@ class AudioSegment(object):
         p_out, p_err = p.communicate()
 
         if p.returncode != 0:
-            raise CouldntEncodeError("Encoding failed. ffmpeg/avlib returned error code: {0}\n\nOutput from ffmpeg/avlib:\n\n{1}".format(p.returncode, p_err))
+            raise CouldntEncodeError("Encoding failed. ffmpeg/avlib returned error code: {0}\n\nCommand:{1}\n\nOutput from ffmpeg/avlib:\n\n{2}".format(p.returncode, conversion_command, p_err))
 
         output.seek(0)
         out_f.write(output.read())
@@ -725,6 +759,61 @@ class AudioSegment(object):
     def duration_seconds(self):
         return self.frame_rate and self.frame_count() / self.frame_rate or 0.0
 
+    def get_dc_offset(self, channel=1):
+        """
+        Returns a value between -1.0 and 1.0 representing the DC offset of a
+        channel (1 for left, 2 for right).
+        """
+        if not 1 <= channel <= 2:
+            raise ValueError("channel value must be 1 (left) or 2 (right)")
+
+        if self.channels == 1:
+            data = self._data
+        elif channel == 1:
+            data = audioop.tomono(self._data, self.sample_width, 1, 0)
+        else:
+            data = audioop.tomono(self._data, self.sample_width, 0, 1)
+
+        return float(audioop.avg(data, self.sample_width)) / self.max_possible_amplitude
+
+    def remove_dc_offset(self, channel=None, offset=None):
+        """
+        Removes DC offset of given channel. Calculates offset if it's not given.
+        Offset values must be in range -1.0 to 1.0. If channel is None, removes
+        DC offset from all available channels.
+        """
+        if channel and not 1 <= channel <= 2:
+            raise ValueError("channel value must be None, 1 (left) or 2 (right)")
+
+        if offset and not -1.0 <= offset <= 1.0:
+            raise ValueError("offset value must be in range -1.0 to 1.0")
+
+        if offset:
+            offset = int(round(offset * self.max_possible_amplitude))
+
+        def remove_data_dc(data, off):
+            if not off:
+                off = audioop.avg(data, self.sample_width)
+            return audioop.bias(data, self.sample_width, -off)
+
+        if self.channels == 1:
+            return self._spawn(data=remove_data_dc(self._data, offset))
+
+        left_channel = audioop.tomono(self._data, self.sample_width, 1, 0)
+        right_channel = audioop.tomono(self._data, self.sample_width, 0, 1)
+
+        if not channel or channel == 1:
+            left_channel = remove_data_dc(left_channel, offset)
+
+        if not channel or channel == 2:
+            right_channel = remove_data_dc(right_channel, offset)
+
+        left_channel = audioop.tostereo(left_channel, self.sample_width, 1, 0)
+        right_channel = audioop.tostereo(right_channel, self.sample_width, 0, 1)
+
+        return self._spawn(data=audioop.add(left_channel, right_channel,
+                                            self.sample_width))
+
     def apply_gain(self, volume_change):
         return self._spawn(data=audioop.mul(self._data, self.sample_width,
                                             db_to_float(float(volume_change))))
