@@ -29,7 +29,7 @@ def apply_mono_filter_to_each_channel(seg, filter_fn):
             index = (sample_i * n_channels) + channel_i
             out_data[index] = sample
 
-    return seg._spawn(out_data.tostring())
+    return seg._spawn(out_data)
 
 
 @register_pydub_effect
@@ -223,8 +223,8 @@ def low_pass_filter(seg, cutoff):
             offset = (i * seg.channels) + j
             last_val[j] = last_val[j] + (alpha * (original[offset] - last_val[j]))
             filteredArray[offset] = int(last_val[j])
-    
-    return seg._spawn(data=filteredArray.tostring())
+
+    return seg._spawn(data=filteredArray)
 
 
 @register_pydub_effect
@@ -256,10 +256,10 @@ def high_pass_filter(seg, cutoff):
 
             last_val[j] = alpha * (last_val[j] + original[offset] - original[offset_minus_1])
             filteredArray[offset] = int(min(max(last_val[j], minval), maxval))
+
+    return seg._spawn(data=filteredArray)
+    
     
-    return seg._spawn(data=filteredArray.tostring())
-
-
 @register_pydub_effect
 def pan(seg, pan_amount):
     """
@@ -318,4 +318,4 @@ def apply_gain_stereo(seg, left_gain=0.0, right_gain=0.0):
     
     return seg._spawn(data=output,
                 overrides={'channels': 2,
-                           'frame_width': 2 * seg.sample_width})
+                           'frame_width': 2 * seg.sample_width})
\ No newline at end of file
